/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.ui.elements

import androidx.compose.foundation.ScrollbarStyle
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.Orientation
import androidx.compose.foundation.gestures.draggable
import androidx.compose.foundation.gestures.rememberDraggableState
import androidx.compose.foundation.gestures.scrollBy
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import com.composables.composetheme.ComposeTheme
import com.composables.composetheme.material3.colorScheme
import edu.ucsc.its.temerity.remote.Device
import kotlinx.coroutines.launch


@Composable
fun DeviceListView(deviceList: List<Device>) {
  val unHoverColor = ComposeTheme.colorScheme.surfaceContainerHighest
  val hoverColor = ComposeTheme.colorScheme.secondaryContainer
  val scrollbarStyle = ScrollbarStyle(
    minimalHeight = 32.dp,
    thickness = 12.dp,
    shape = RoundedCornerShape(8.dp),
    hoverDurationMillis = 160,
    unhoverColor = unHoverColor,
    hoverColor = hoverColor,
  )

  val scrollState = rememberLazyListState()
  val coroutineScope = rememberCoroutineScope()

  Box(modifier = Modifier.fillMaxSize()) {
    LazyColumn(
      state = scrollState,
      modifier = Modifier
        .padding(end = 12.dp)
        .draggable(
          orientation = Orientation.Horizontal,
          state = rememberDraggableState { delta ->
            coroutineScope.launch {
              scrollState.scrollBy(-delta)
            }
          },
        ),
    ) {
      items(deviceList.size) { index ->
        DeviceCardRow(deviceList[index])
      }
    }
    VerticalScrollbar(
      adapter = rememberScrollbarAdapter(scrollState),
      modifier = Modifier
        .align(Alignment.CenterEnd)
        .fillMaxHeight(),
      style = scrollbarStyle,
    )
  }
}

@Composable
fun DeviceCardRow(device: Device) {
  val textColor = ComposeTheme.colorScheme.primary
  val spacerColor = ComposeTheme.colorScheme.background

  GradientCard {
    Row(
      verticalAlignment = Alignment.CenterVertically,
      modifier = Modifier
        .fillMaxSize()
        .heightIn(min = 64.dp)
        .padding(8.dp),

    ) {
      Column {
        Text(
          text = device.stationName,
          color = textColor,
        )
        Spacer(
          modifier = Modifier
            .height(8.dp)
            .background(spacerColor),
        )
        Text(
          text = device.stationType,
          color = textColor,
        )
      }
    }
  }
}

@Composable
fun DeviceDetailsView(device: Device) {
  
}
