package edu.ucsc.its.temerity.shared.ui.elements

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.AdUnits
import androidx.compose.material.icons.outlined.Contacts
import androidx.compose.material.icons.outlined.Edit
import androidx.compose.material.icons.outlined.Notifications
import androidx.compose.material.icons.outlined.Settings
import androidx.compose.material.icons.rounded.Clear
import androidx.compose.material.icons.rounded.Search
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationBar
import androidx.compose.material3.NavigationBarItem
import androidx.compose.material3.NavigationDrawerItem
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.OutlinedTextFieldDefaults
import androidx.compose.material3.PermanentDrawerSheet
import androidx.compose.material3.PermanentNavigationDrawer
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.minimumInteractiveComponentSize
import androidx.compose.material3.windowsizeclass.ExperimentalMaterial3WindowSizeClassApi
import androidx.compose.material3.windowsizeclass.WindowWidthSizeClass
import androidx.compose.material3.windowsizeclass.calculateWindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.navigation.NavHostController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.createGraph
import com.composables.composetheme.ComposeTheme
import com.composables.composetheme.material3.colorScheme
import edu.ucsc.its.temerity.PlatformClient
import edu.ucsc.its.temerity.remote.Device
import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.withContext
import org.dotenv.vault.dotenvVault
import org.jetbrains.compose.resources.StringResource
import org.koin.compose.getKoin
import org.koin.core.parameter.parametersOf
import temerity.shared.compose.generated.resources.Res
import temerity.shared.compose.generated.resources.devices
import temerity.shared.compose.generated.resources.login
import temerity.shared.compose.generated.resources.main
import temerity.shared.compose.generated.resources.notifications
import temerity.shared.compose.generated.resources.roster
import temerity.shared.compose.generated.resources.settings

enum class MainContentWindow(val title: StringResource) {
    Notifications(title = Res.string.notifications),
    Devices(title = Res.string.devices),
    Roster(title = Res.string.roster),
    Settings(title = Res.string.settings)
}

enum class MainDetailWindow(val title: StringResource) {

}

@OptIn(ExperimentalMaterial3WindowSizeClassApi::class)
@Composable
fun ListDetailLayoutWithSearchBar() {
    // Set initial main content pane to "Devices"
    var navigationIndex by remember { mutableStateOf(1) }
    val selectedDeviceIndex by remember { mutableStateOf(0) }
    val widthSizeClass = calculateWindowSizeClass().widthSizeClass

    val dotenv = dotenvVault(){
        directory = "../temerity"
    }
    val pc: PlatformClient = getKoin().get(parameters = { parametersOf(dotenv["YUJAPROD_API_URL"], dotenv["YUJAPROD_TOKEN"]) })
    var devicesList by remember { mutableStateOf(listOf<Device>()) }

    LaunchedEffect(pc) {
        withContext(IO) {
            devicesList = pc.getDevices().sortedBy { it.stationName }
        }
    }

    val navController = rememberNavController()
    val navGraph = navController.createGraph(
        startDestination = MainContentWindow.Devices.name
    ) {
        composable(MainContentWindow.Notifications.name) {
            // NotificationsScreen()
        }
        composable(MainContentWindow.Devices.name) {
            DeviceListView(devicesList)
        }
        composable(MainContentWindow.Roster.name) {
            // RosterScreen()
        }
        composable(MainContentWindow.Settings.name) {
            // SettingsScreen()
        }
    }

    val navigationItems = listOf(
        mapOf("icon" to Icons.Outlined.Notifications, "label" to "Notifications"),
        mapOf("icon" to Icons.Outlined.AdUnits, "label" to "Devices"),
        mapOf("icon" to Icons.Outlined.Contacts, "label" to "Roster"),
        mapOf("icon" to Icons.Outlined.Settings, "label" to "Settings"),
    )

    @Composable
    fun SearchBar(
        modifier: Modifier = Modifier,
        searchQuery: String,
        onSearchQueryChange: (String) -> Unit,
        onImeSearch: () -> Unit,
    ) {
        Box(modifier = modifier.padding(horizontal = 16.dp, vertical = 10.dp)) {
            Row(horizontalArrangement = Arrangement.spacedBy(8.dp), verticalAlignment = Alignment.CenterVertically) {
                OutlinedTextField(colors = OutlinedTextFieldDefaults.colors(
                    unfocusedContainerColor = ComposeTheme.colorScheme.surface,
                    focusedContainerColor = ComposeTheme.colorScheme.surface,
                ), shape = RoundedCornerShape(100), placeholder = { Text("Search...") }, leadingIcon = {
                    Icon(imageVector = Icons.Rounded.Search, contentDescription = null, modifier = Modifier.minimumInteractiveComponentSize(), tint = ComposeTheme.colorScheme.onSurface.copy(alpha = 0.66f))
                }, modifier = Modifier.weight(1f).minimumInteractiveComponentSize(), value = searchQuery, singleLine = true, keyboardActions = KeyboardActions(onSearch = { onImeSearch() }), keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text, imeAction = ImeAction.Search), onValueChange = onSearchQueryChange, trailingIcon = {
                    AnimatedVisibility(visible = searchQuery.isNotBlank(), enter = fadeIn(), exit = fadeOut()) {
                        IconButton(onClick = { onSearchQueryChange("") }) {
                            Icon(imageVector = Icons.Rounded.Clear, contentDescription = "Clear", tint = MaterialTheme.colorScheme.onSurface)
                        }
                    }
                })
            }
        }
    }

    @Composable
    fun ContentPanel(modifier: Modifier = Modifier) {
        var searchQuery by remember { mutableStateOf("") }
        Column(modifier) {
            SearchBar(
                searchQuery = searchQuery,
                onSearchQueryChange = { searchQuery = it },
                onImeSearch = { /*TODO*/ },
            )
            Box(Modifier.weight(1f).fillMaxWidth().padding(16.dp).border(1.dp, ComposeTheme.colorScheme.outline, RoundedCornerShape(18.dp)).clip(RoundedCornerShape(18.dp)).background(ComposeTheme.colorScheme.surface)) {
                NavHost(
                    navController = navController,
                    graph = navGraph,
                )
            }
        }
    }

    @Composable
    fun DetailPanel(modifier: Modifier = Modifier) {
        Box(modifier.fillMaxHeight().padding(16.dp).border(1.dp, ComposeTheme.colorScheme.outline, RoundedCornerShape(18.dp)).clip(RoundedCornerShape(18.dp)).background(ComposeTheme.colorScheme.surface)) {

        }
    }

    if (widthSizeClass == WindowWidthSizeClass.Compact) {
        Scaffold(modifier = Modifier.fillMaxHeight(), floatingActionButton = {
            FloatingActionButton(onClick = { /* TODO */ }) {
                Icon(Icons.Outlined.Edit, "Compose new message")
            }
        }, bottomBar = {
            NavigationBar {
                navigationItems.forEachIndexed { index, _ ->
                    val item = navigationItems[index]
                    val icon = item["icon"] as ImageVector
                    val label = item["label"] as String
                    NavigationBarItem(selected = index == navigationIndex, icon = { Icon(icon, null) }, label = { Text(label) }, onClick = { navigationIndex = index; navController.navigate(label.lowercase()) })
                }
            }
        }) { padding ->
            ContentPanel(Modifier.fillMaxSize().padding(padding))
        }
    } else {
        Scaffold(modifier = Modifier.fillMaxHeight()) {
            PermanentNavigationDrawer(drawerContent = {
                Row {
                    PermanentDrawerSheet(Modifier.width(220.dp).padding(vertical = 24.dp)) {
                        Column {
                            ExtendedFloatingActionButton(
                                onClick = { /*TODO*/ },
                                modifier = Modifier.padding(horizontal = 12.dp),
                            ) {
                                Row(verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(8.dp, Alignment.Start)) {
                                    Icon(Icons.Outlined.Edit, null)
                                    Text("Compose")
                                }
                            }
                            Spacer(Modifier.height(24.dp))
                            LazyColumn {
                                itemsIndexed(navigationItems) { index, _ ->
                                    val item = navigationItems[index]
                                    val icon = item["icon"] as ImageVector
                                    val label = item["label"] as String
                                    NavigationDrawerItem(modifier = Modifier.padding(horizontal = 16.dp), selected = index == navigationIndex, icon = { Icon(icon, null) }, label = { Text(label) }, onClick = { navigationIndex = index; navController.navigate(label.lowercase()) })
                                }
                            }
                        }
                    }
                    HorizontalDivider(modifier = Modifier.width(1.dp).fillMaxHeight())
                }
            }, content = {
                Row {
                    ContentPanel(Modifier.weight(1f).fillMaxHeight())
                    if (widthSizeClass == WindowWidthSizeClass.Expanded) {
                        HorizontalDivider(modifier = Modifier.width(1.dp).fillMaxHeight())
                        DetailPanel(Modifier.weight(1f).padding(top = 76.dp))
                    }
                }
            })
        }
    }
}