/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity

import edu.ucsc.its.temerity.remote.PlatformClientImpl
import edu.ucsc.its.temerity.remote.AuditLogEntry
import edu.ucsc.its.temerity.remote.Course
import edu.ucsc.its.temerity.remote.Device
import edu.ucsc.its.temerity.remote.DeviceRecordingSession
import edu.ucsc.its.temerity.remote.FolderPermissions
import edu.ucsc.its.temerity.remote.Group
import edu.ucsc.its.temerity.remote.NewUser
import edu.ucsc.its.temerity.remote.User
import edu.ucsc.its.temerity.remote.UserGroup
import edu.ucsc.its.temerity.remote.UserRecordingSession
import edu.ucsc.its.temerity.remote.UserUpdate
import io.ktor.client.statement.HttpResponse
import kotlinx.datetime.DatePeriod
import kotlinx.datetime.DateTimePeriod
import kotlinx.datetime.DateTimeUnit
import kotlinx.datetime.LocalDate
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.format
import kotlinx.datetime.format.DateTimeFormat
import kotlinx.datetime.format.char
import kotlinx.datetime.plus
import kotlinx.serialization.KSerializer
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder

/**
 * This interface defines top-level library functions for interacting with the enterprise video platform API.
 * The functions defined in this interface are implemented by the [PlatformClientImpl] class.
 * @see [PlatformClientImpl]
 */

public interface PlatformClient {

  // User API public library functions

  /**
   * Fetches the list of users from the platform.
   * @return A list of User objects
   * @throws RuntimeException for:
   *  - Network issues
   *  Includes HttpResponse if the request fails due to:
   *  - Invalid service endpoint (404 not found)
   *  - Invalid service token (401 unauthorized)
   */
  public suspend fun getUsers(): List<User>

  /**
   * Fetches a specific user from the platform.
   * @param userId The ID of the user to fetch.
   * @return A list of User objects.
   * @throws RuntimeException for:
   *  - Network issues
   *  Includes HttpResponse if the request fails due to:
   *  - Invalid service endpoint (404 not found)
   *  - Invalid service token (401 unauthorized)
   */
  public suspend fun getUser(userId: Long): User

  /**
   * Creates a new user on the platform.
   * @param primaryIdentifier An optional primary identifier attribute for the user, such as an email address or login ID.
   * Used to uniquely identify the user in the platform when logging in from an external source, such as LMS.
   * @param newUser A NewUser object representing the details of the user to create, some of which are required.
   * @see <a href="https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKJ0GKB8J5TMZ3DFA4V7">Required new user fields</a>
   * @return HttpResponse object indicating success or failure of the request.
   * @throws RuntimeException for:
   *  - Network issues
   *  Includes HttpResponse if the request fails due to:
   *  - Invalid NewUser object (415 unsupported media type)
   *  - Invalid service endpoint (404 not found)
   *  - Invalid service token (401 unauthorized)
   */
  public suspend fun createUser(primaryIdentifier: String = "", newUser: NewUser): HttpResponse

  /**
   * Updates a specific user on the platform.
   * @param userId The unique platform ID of the user record to write.
   * @param userUpdate A UserUpdate object representing only the detail fields you wish to update of a specific user.
   * @see <a href="https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKJ0JRN8T6G7QNMHP15F">All optional user update fields</a>
   * @return HttpResponse object indicating success or failure of the request.
   * @throws RuntimeException for:
   *  - Network issues
   *  Includes HttpResponse if the request fails due to:
   *  - Invalid service endpoint (404 not found)
   *  - Invalid service token (401 unauthorized)
   */
  public suspend fun updateUser(userId: Long, userUpdate: UserUpdate): HttpResponse

  /**
   * Deletes a specific user from the platform.
   * @param userId The ID of the user to delete.
   * @return HttpResponse object indicating success or failure of the request:
   *  - 200 OK if the user was successfully deleted, with payload:
   *    "Deleted user: $userId success."
   *  - 404 Not Found if the user does not exist, with payload:
   *    "Error - No such user."
   * @throws RuntimeException for:
   *  - Network issues
   *  Includes HttpResponse if the request fails due to:
   *  - Invalid service endpoint (404 not found)
   *  - Invalid service token (401 unauthorized)
   */
  public suspend fun deleteUser(userId: Long): HttpResponse

  /**
   * Fetches a list of groups that a user is a member of.
   * @param userId The ID of the user to fetch the groups for.
   * @return A list of UserGroup objects.
   * @throws Exception if the groups cannot be fetched.
   */
  public suspend fun getUserGroups(userId: Long): List<UserGroup>

  /**
   * Fetches a list of groups that a user is the content owner of.
   * @param userId The ID of the user to fetch the groups for.
   * @return A list of UserGroup objects.
   * @throws Exception if the groups cannot be fetched.
   */
  public suspend fun getUserGroupsOwned(userId: Long): List<UserGroup>

  public suspend fun createUserWithExternalId(primaryIdentifier: String, newUser: NewUser): HttpResponse

  public suspend fun addFolderPermission(perms: FolderPermissions.Action): HttpResponse

  public suspend fun removeFolderPermission(perms: FolderPermissions.Action)

  public suspend fun getMediaMetadata(linkageValue: String)

  // Group API public library functions

  public suspend fun getGroups(): List<Group>

  public suspend fun getCourse(courseCode: String): Course

  public suspend fun getCourses(): List<Course>

  /**
   * Fetches a list of audit log entries from the platform for a given time period and list of event types.
   * @param startTime The start date for the log entries.
   * @param endTime The end date for the log entries.
   * @param eventTypeList A list of event types to filter the log entries.
   * @param sortOrder The order in which to sort the log entries.
   * @return A list of AuditLogEntry objects.
   */
  public suspend fun getAuditLogEntries(
    startTime: LocalDate,
    endTime: LocalDate,
    eventTypeList: List<EventType>? = null,
    sortOrder: AuditLogSortOrder? = null,
  ): List<AuditLogEntry>

  /**
   * Fetches a list of audit log entries from the platform for a given time period and a single event type.
   * @param startTime The start date for the log entries.
   * @param endTime The end date for the log entries.
   * @param eventType A single event type to filter for among the log entries.
   * @param sortOrder The order in which to sort the log entries.
   * @return A list of AuditLogEntry objects.
   */
  public suspend fun getAuditLogEntries(
    startTime: LocalDate,
    endTime: LocalDate,
    eventType: EventType,
    sortOrder: AuditLogSortOrder? = null,
  ): List<AuditLogEntry>

  /**
   * Fetches a list of devices registered with the current instance from the platform.
   * @return A list of Device objects.
   */
  public suspend fun getDevices(): List<Device>

  /**
   * Fetches a specific device from the platform.
   * @param deviceId The ID of the device to fetch.
   * @return A Device object.
   */
  public suspend fun getDevice(deviceId: Long): Device

  /**
   * Fetches a list of sessions a user has scheduled for a given time period.
   * @param userId The ID of the user to fetch the sessions for.
   * @param startTime The start date for the sessions.
   * @param endTime The end date for the sessions.
   * @return A list of UserRecordingSession objects.
   */
  public suspend fun getUserSessions(userId: String, startTime: LocalDate, endTime: LocalDate): List<UserRecordingSession>

  /**
   * Fetches a list of sessions on a device's schedule for a given time period.
   * @param deviceId The ID of the device to fetch the sessions for.
   * @param startTime The start date for the sessions. Defaults to the current date.
   * @param endTime The end date for the sessions. Defaults to one week from the current date.
   * @return A list of DeviceRecordingSession objects.
   */
  public suspend fun getDeviceSchedule(deviceId: Long, startTime: LocalDate = currentDate(), endTime: LocalDate = currentDate().plus(1, DateTimeUnit.WEEK)): List<DeviceRecordingSession>

  /**
   * Fetches a file containing storage usage details of a specific group.
   * @param groupId The ID of the group to fetch the storage usage for.
   */
  public suspend fun getStorageAnalyticsReport(groupId: Long): ByteArray

  /**
   * This interface represents the default PlatformClient builder pattern.
   * It provides methods to set the service endpoint and token, and to build the PlatformClient.
   */
  public interface Builder {
    public fun debugEnabled(debugEnabled: Boolean)

    public fun serviceEndpoint(endpoint: String) {}
    public fun serviceToken(token: String) {}
    public fun build(): PlatformClient {
      TODO()
    }

    @Suppress("CanBePrivate")
    public fun build(builder: Builder.() -> Unit): PlatformClient
  }
}

// Objects used by implementations of Temerity's PlatformClient

private val AUDIT_LOG_REQUEST_DATE_FORMAT: DateTimeFormat<LocalDate> = LocalDate.Format {
  dayOfMonth()
  char('/')
  monthNumber()
  char('/')
  year()
}
private val AUDIT_LOG_TIMESTAMP_FORMAT: DateTimeFormat<LocalDateTime> = LocalDateTime.Format {
  year()
  char('-')
  monthNumber()
  char('-')
  dayOfMonth()
  char(' ')
  hour()
  char(':')
  minute()
  char(':')
  second()
}
private val SCHEDULED_SESSION_DATE_FORMAT: DateTimeFormat<LocalDate> = LocalDate.Format {
  dayOfMonth()
  char('-')
  monthNumber()
  char('-')
  year()
}

/**
 * Enum class of YuJa API Audit Log event types
 *
 * This class is used to represent the different types of events that can be queried from the YuJa API's audit log endpoint.
 * @param value The string value of the event type as it is represented in the YuJa API. Used to make the API call.
 */
public enum class EventType(public val value: String) {
  ACTIVE_ARCHIVING_REQUEST("Active Archiving Request"),
  AD_ASTRA_SYNC_FAILED("Ad Astra Sync Failed"),
  ALL_CAPTIONS("All Captions"),
  ALL_USER_NOTIFICATIONS("All User Notifications"),
  ANONYMOUS_USER_QUIZ_ATTEMPT_STARTED("Anonymous User Quiz Attempt Started"),
  ANONYMOUS_USER_QUIZ_QUESTION_ANSWERED("Anonymous User Quiz Question Answered"),
  ANONYMOUS_USER_QUIZ_QUESTION_REACHED("Anonymous User Quiz Question Reached"),
  ANONYMOUS_USER_QUIZ_SUBMISSION("Anonymous User Quiz Submission"),
  AUTO_CAPTIONS("Auto-Captions"),
  AUTOMATED_SESSION_FAILED_TO_START("Automated Session failed to Start"),
  AUTOMATED_SESSION_MONITOR("Automated Session Monitor"),
  BLACKBOARD_COLLABORATE_IMPORTING("Blackboard Collaborate Importing"),
  BULK_BLACKBOARD_USER_IMPORT("Bulk Blackboard User Import"),
  BULK_WEBEX_USER_IMPORT("Bulk Webex User Import"),
  BULK_ZOOM_USER_IMPORT("Bulk Zoom User Import"),
  CAPTURE_ERROR("Capture Error"),
  CHANGE_OWNER("Change Owner"),
  CHANGE_THUMBNAIL("Change Thumbnail"),
  COMMENTS("Comments"),
  COURSES_AND_GROUPS("Courses and Groups"),
  CREATE_QUIZ("Create Quiz"),
  DELETE_CAPTIONS("Delete Captions"),
  DELETE_FOLDER("Delete Folder"),
  DELETE_MEDIA("Delete Media"),
  DELETE_QUIZ("Delete Quiz"),
  DELETE_TRANSCRIPT("Delete Transcript"),
  DELETE_USER("Delete User"),
  DEVICE_ERROR("Device Error"),
  DEVICE_FAILED_TO_START("Device Failed to Start"),
  DEVICE_STATUS_AND_ERROR_LOGS("Device Status And Error Logs"),
  DEVICE_STATUS("DEVICE_STATUS"),
  DOWNLOAD_CONTENT("Download Content"),
  DOWNLOAD_REQUEST("Download Request"),
  EDIT_MEDIA("Edit Media"),
  EDIT_SCHEDULE("Edit Schedule"),
  ERROR("ERROR"),
  Event("Event"),
  FAILED_LOG_IN("Failed Log In"),
  FAVORITE_MEDIA("Favorite Media"),
  FOLDER_MANAGEMENT("Folder Management"),
  GENERATE_PERMALINK("Generate Permalink"),
  GRADE_NOT_SENT_TO_LMS_GRADEBOOK("Grade Not Sent to LMS Gradebook"),
  HARDWARE_CAPTURE_HUB("Hardware Capture (HUB)"),
  HARDWARE_CAPTURE_HUB_CAPTURE_INITIATED("Hardware Capture (HUB) - Capture Initiated"),
  HDCP_SIGNAL_DETECTED("HDCP Signal Detected"),
  HIMALAYAS("Himalayas"),
  HIMALAYAS_ARCHIVING("Himalayas Archiving"),
  HIMALAYAS_RESTORE("Himalayas Restore"),
  HUB_CAPTURE_PROBLEM("Hub Capture Problem"),
  HUMAN_CAPTION_NEEDS_APPROVAL("Human Caption Needs Approval"),
  HUMAN_CAPTION_FINISHED_FROM_YUJA_PRO("Human-Caption Finished from Yuja Pro Captioning"),
  HUMAN_CAPTION_REJECTED_BY_YUJA_PRO("Human-Caption Rejected by Yuja Pro Captioning"),
  HUMAN_CAPTION_STARTED_FROM_YUJA_PRO("New Human-Caption Started from YuJa Pro Captioning"),
  HUMAN_CAPTIONS("Human-Captions"),
  IMPORTING_ZOOM_RECORDING("Importing Zoom Recording"),
  INTEGRATIONS("Integrations"),
  LMS_GRADEBOOK_FAILED_TO_SYNC("LMS Gradebook Failed to Sync"),
  LMS_GRADEBOOK_SUCCESSFULLY_SYNCED("LMS Gradebook Successfully Synced"),
  LMS_USER_QUIZ_ATTEMPT_STARTED("LMS User Quiz Attempt Started"),
  LMS_USER_QUIZ_QUESTION_ANSWERED("LMS User Quiz Question Answered"),
  LMS_USER_QUIZ_QUESTION_REACHED("LMS User Quiz Question Reached"),
  LMS_USER_QUIZ_SUBMISSION("LMS User Quiz Submission"),
  MEDIA_DOWNLOADS("Media Downloads"),
  MEDIA_MANAGEMENT("Media Management"),
  MEDIA_PLAYER_BRANDING("Media Player Branding"),
  MICROSOFT_TEAMS_IMPORTING("Microsoft Teams Importing"),
  MOBILE_DOWNLOADS("Mobile Downloads"),
  MOVE_FOLDER("Move Folder"),
  MOVE_MEDIA("Move Media"),
  NEW_ALERT("New Alert"),
  NEW_AUTO_CAPTION("New Auto-Caption"),
  NEW_BROWSER_CAPTURE("New Browser Capture"),
  NEW_CAPTURE("New Capture"),
  NEW_COURSE("New Course"),
  NEW_FOLDER("New Folder"),
  NEW_HUMAN_CAPTION_STARTED_FROM_YUJA_PRO("New Human-Caption Started from YuJa Pro Captioning"),
  NEW_LOG_IN("New Log In"),
  NEW_MEDIA("New Media"),
  NEW_MOBILE_LOG_IN("New Mobile Log In"),
  NEW_PUBLISH("New Publish"),
  NEW_SCHEDULE("New Schedule"),
  NEW_USER("New User"),
  NEW_VIDEO_CONFERENCE("New Video Conference"),
  NEW_VIEW("New View"),
  POST_QUIZ("Post Quiz"),
  PUBLISH_QUIZ("Publish Quiz"),
  QUIZ_GRADE_FOR_LMS_CALCULATED("Quiz Grade for LMS Calculated"),
  RECORDING_AND_STREAMING("Recording and Streaming"),
  RECORDING_ERROR("Recording Error"),
  RECURRING_ACTION_PERFORMED("Recurring Action Performed"),
  RECYCLE_FOLDER("Recycle Folder"),
  RECYCLE_MEDIA("Recycle Media"),
  RENAME_FOLDER("Rename Folder"),
  RENAME_MEDIA("Rename Media"),
  REPLACE_MEDIA("Replace Media"), // Not documented, but returned by API
  RESTORE_ACTIVE_ARCHIVE("Restore Active Archive"),
  RESTORE_FOLDER("Restore Folder"),
  RESTORE_MEDIA("Restore Media"),
  SAML_SSO_SETTINGS_UPDATED("SAML SSO Settings Updated"),
  SESSION("SESSION"),
  SHARE_FOLDER("Share Folder"),
  SHARE_MEDIA("Share Media"),
  SHARE_MEDIA_ASSET("Share Media Asset"),
  SOFTWARE_CAPTURE_MAC("Software Capture (Mac)"),
  SOFTWARE_CAPTURE_MAC_CAPTURE_INITIATED("Software Capture (Mac) - Capture Initiated"),
  SOFTWARE_CAPTURE_PC("Software Capture (PC)"),
  SOFTWARE_CAPTURE_PC_CAPTURE_INITIATED("Software Capture (PC) - Capture Initiated"),
  SUB_CLIPS("Sub-Clips"),
  UNDO_ACTIVE_ARCHIVE("Undo Active Archive"),
  UNFAVORITE_MEDIA("Unfavorite Media"),
  UNPUBLISH_FOLDER("Unpublish Folder"),
  UNPUBLISH_MEDIA("Unpublish Media"),
  UPLOAD_CAPTIONS("Upload Captions"),
  UPLOAD_TRANSCRIPT("Upload Transcript"),
  USER_CREATED_EMPTY_POLICY("User Created Empty Policy"),
  USER_DEMOTED("User Demoted"),
  USER_PROMOTED("User Promoted"),
  USER_QUIZ_QUESTION_ANSWERED("User Quiz Question Answered"),
  USER_QUIZ_QUESTION_REACHED("User Quiz Question Reached"),
  USER_QUIZ_SUBMISSION("User Quiz Submission"),
  USERS("Users"),
  VIDEO_CONFERENCE("Video Conference"),
  VIDEO_QUIZ("Video Quiz"),
  VIEWER_DOWNLOADS("Viewer Downloads"),
  WEBEX_IMPORTING("Webex Importing"),
  ZOOM_IMPORT_FAILED("Zoom Import Failed"),
  ZOOM_IMPORTING("Zoom Importing"),
  ZOOM_VERIFICATION("Zoom Verification"),
  WARNING("WARNING"), // Not documented, but returned by API
}

/**
 * Exception class used to break out of a loop in the [PlatformClientImpl] class when reading paginated API responses.
 */
internal class BreakException(val page: Int) : RuntimeException() {
  fun lastPage(): Int = page
}

/**
 * Enum class used to define the sort order for audit log entries.
 */
public enum class AuditLogSortOrder {
  NEW_FIRST,
  OLD_FIRST,
  ;

  public companion object {
    public fun fromBoolean(value: Boolean): AuditLogSortOrder {
      return if (value) NEW_FIRST else OLD_FIRST
    }
  }
}

/**
 * Serializer for LocalDateTime objects used in the [AuditLogEntry] class.
 * This object is used to serialize and deserialize LocalDateTime objects to and from strings in the format used by the YuJa API.
 */
internal object LocalDateTimeSerializer : KSerializer<LocalDateTime> {
  override val descriptor: SerialDescriptor = PrimitiveSerialDescriptor("LocalDateTime", PrimitiveKind.STRING)

  override fun serialize(encoder: Encoder, value: LocalDateTime) {
    encoder.encodeString(value.format(AUDIT_LOG_TIMESTAMP_FORMAT))
  }

  override fun deserialize(decoder: Decoder): LocalDateTime {
    return LocalDateTime.parse(decoder.decodeString(), AUDIT_LOG_TIMESTAMP_FORMAT)
  }
}

/**
 * Extension functions used by Temerity's PlatformClient
 */

/**
 * Formats a [LocalDate] object as a string in the format used by the YuJa API for audit log requests.
 * @return A string representation of the date in the format "dd/MM/yyyy".
 */
internal fun LocalDate.applyAuditLogFormat(): String =
  format(AUDIT_LOG_REQUEST_DATE_FORMAT)

/**
 * Formats a [LocalDate] object as a string in the format used by the YuJa API for sessions.
 * @return A string representation of the date in the format "dd-MM-yyyy".
 */
internal fun LocalDate.applyScheduledSessionDateFormat(): String =
  format(SCHEDULED_SESSION_DATE_FORMAT)

/**
 * Sorts a list of [AuditLogEntry] objects by their creation date.
 * @param sortOrder The order in which to sort the entries.
 */
public fun ArrayList<AuditLogEntry>.sortByCreationDate(sortOrder: AuditLogSortOrder) {
  when (sortOrder) {
    AuditLogSortOrder.NEW_FIRST -> sortByDescending { auditLogEntry -> auditLogEntry.creationTimestamp }
    AuditLogSortOrder.OLD_FIRST -> sortBy { auditLogEntry -> auditLogEntry.creationTimestamp }
  }
}
