/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.remote

import com.skydoves.sandwich.ApiResponse
import de.jensklingenberg.ktorfit.http.Body
import de.jensklingenberg.ktorfit.http.DELETE
import de.jensklingenberg.ktorfit.http.GET
import de.jensklingenberg.ktorfit.http.POST
import de.jensklingenberg.ktorfit.http.PUT
import de.jensklingenberg.ktorfit.http.Path
import de.jensklingenberg.ktorfit.http.Query
import de.jensklingenberg.ktorfit.http.Streaming
import io.ktor.client.statement.HttpResponse
import io.ktor.client.statement.HttpStatement

internal interface PlatformApi {

  /**
   * User API Endpoints
   * https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKHZB4Q5S9GC1P7WWGG8
   */
  @Streaming
  @GET("users")
  suspend fun getUsers(): HttpStatement

  @Streaming
  @GET("users/{userId}")
  suspend fun getUser(@Path userId: Long): HttpStatement

  @POST("users/{primary_identifier}")
  suspend fun createUser(
    @Path("primary_identifier") primaryIdentifier: String,
    @Body newUser: String,
  ): ApiResponse<HttpResponse>

  @PUT("users/{userId}")
  suspend fun setUser(@Path userId: Long, @Body user: String): ApiResponse<HttpResponse>

  @DELETE("users/{userId}")
  suspend fun deleteUser(@Path userId: Long): ApiResponse<HttpResponse>

  @Streaming
  @GET("users/{userId}/groups/members")
  suspend fun getUserGroups(@Path userId: Long): HttpStatement

  @Streaming
  @GET("users/{userId}/groups/content_owners")
  suspend fun getUserGroupsOwned(@Path userId: Long): HttpStatement

  @POST("services/media/permission")
  suspend fun addFolderPermission(
    @Body addFolderPermissions: String,
  ): ApiResponse<HttpResponse>

  @POST("services/media/permission")
  suspend fun removeFolderPermission(
    @Body deleteFolderPermissions: String,
  ): ApiResponse<HttpResponse>

  /**
   * Group API Endpoints
   * https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKJ1YPHZ5KBP9G8GCJRD
   */
  @Streaming
  @GET("groups")
  suspend fun getGroupPage(
    @Query("page") page: Int,
    @Query("sortDescending") sortDescending: Boolean,
  ): HttpStatement

  @GET("courses/{courseCode}")
  suspend fun getCourse(@Path courseCode: String): HttpStatement

  @Streaming
  @GET("courses")
  suspend fun getCourses(): HttpStatement

  /**
   * Device API Endpoints
   * https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKJ4QZYVAXT743026AQ2
   */
  @Streaming
  @GET("devices/")
  suspend fun getDevices(): HttpStatement

  @Streaming
  @GET("devices/{deviceId}")
  suspend fun getDeviceById(@Path deviceId: Long): HttpStatement

  @PUT("devices/{deviceId}")
  suspend fun updateDeviceRegistration(@Path deviceId: Long, @Body device: String): ApiResponse<HttpResponse>

  @POST("devices/{deviceId}/register")
  suspend fun registerDevice(@Path deviceId: Long, @Body device: String): ApiResponse<HttpResponse>

  @GET("devices/registrations/{access_id}")
  suspend fun getDeviceRegistration(
    @Query("access_id") deviceCode: Int,
  ): ApiResponse<HttpResponse>

  @Streaming
  @GET("devices/{deviceId}/schedule")
  suspend fun getDeviceSchedule(
    @Path("deviceId") deviceId: Long,
    @Query("startTime") startTime: String?,
    @Query("endTime") endTime: String?,
  ): HttpStatement

  // 4.2.20 - Get Sessions by User
  @Streaming
  @GET("devices/{userId}/sessions")
  suspend fun getUserSessions(
    @Path userId: String,
    @Query("startTime") startTime: String,
    @Query("endTime") endTime: String,
  ): HttpStatement

  @Streaming
  @GET("auditLog/entries")
  suspend fun getAuditLogEntries(
    @Query("startTime") startTime: String?,
    @Query("endTime") endTime: String?,
    @Query("eventType") eventType: String,
    @Query("offset") entryOffset: Int? = 0,
  ): HttpStatement

  @Streaming
  @GET("analytics/report/storage/{groupId}")
  suspend fun getStorageAnalyticsReport(@Path("groupId") groupId: Long): HttpStatement
}
